;+
; Copyright © 2008, United States Government as represented by the
; Administrator for The National Aeronautics and Space Administration.
; All Rights Reserved.
;
; FUNCTION FMTDATETIME(year, month, day, hour, minute, second, TZONE=tzone)
;
; DESCRIPTION
; Formats a datetime string from the individual components of the date and time.
; Optionally appends the timezone specification to the string for use in
; creating a database-compatible, fully-qualified datetime string.  The tzone
; parameter is the number of hours to be added to the datetime to convert it to
; UTC time.  For example, for a time in CST, tzone is 6, since UTC is 6 hours
; ahead of Central Standard Time.
;
; FORMATTING RULES FOR CENTURY according to 'year' parameter are as follows:
;
;   - if a 4-digit year is provided, it is used as-is
;   - if a 2-digit year is 70 or greater, 20th Century is assumed (e.g., 1978)
;   - if year is less than 70, 21st Century is assumed (e.g., 2023)
;   - if a >4-digit, 3-digit, or negative year is provided, ugliness ensues
;
; VALIDITY CHECKING is not done -- the input parameters are assumed to be
; valid date and time and time zone values.  ALL POSITIONAL PARAMETERS ARE
; REQUIRED AS INPUT (year through second).  Time Zone is the only optional
; parameter.
;
; EMAIL QUESTIONS OR COMMENTS TO:
;       <Bob Morris> kenneth.r.morris@nasa.gov
;       <Matt Schwaller> mathew.r.schwaller@nasa.gov
;-

function fmtdatetime, year, month, day, hour, minute, second, TZONE=tzone

    numParams = N_Params()
    dtimestring = ''
    if numParams lt 6 then GOTO, ErrorExit
    zero = '0'
    twenty = '20'
    nineteen = '19'

    if year gt 999 then begin
       yyyy = string(year,format='(i4)')
    endif else begin
       if year ge 70 then begin
          yyyy = nineteen+string(year,format='(i2)')
       endif else begin
          if year lt 10 then yyyy = twenty+zero+string(year,format='(i1)')
          if year ge 10 then yyyy = twenty+string(year,format='(i2)')
       endelse
    endelse

    if month lt 10 then mm = zero+string(month,format='(i1)')
    if month ge 10 then mm = string(month,format='(i2)')

    if day lt 10 then dd = zero+string(day,format='(i1)')
    if day ge 10 then dd = string(day,format='(i2)')

;    print, "mo day hour min sec: ", mm, dd, hour, minute, second
    if hour lt 10 then hh = zero+string( hour,format='(i1)')
    if hour ge 10 then hh = string( hour,format='(i2)')
    if minute lt 10 then mins = zero+string( minute,format='(i1)')
    if minute ge 10 then mins = string( minute,format='(i2)')
    if second lt 10 then ss = zero+string( second,format='(i1)')
    if second ge 10 then ss = string( second,format='(i2)')
    dtimestring = yyyy+"-"+mm+"-"+dd+" "+hh+":"+mins+":"+ss

    if N_Elements(tzone) EQ 1 then begin
       if tzone lt 0 then begin
          tzone = abs(tzone)
          if tzone lt 10 then tzstr = '-'+zero+string(tzone,format='(i1)')
          if tzone ge 10 then tzstr = '-'+string(tzone,format='(i1)')
       endif else begin
          if tzone lt 10 then tzstr = '+'+zero+string(tzone,format='(i1)')
          if tzone ge 10 then tzstr = '+'+string(tzone,format='(i1)')
       endelse
    dtimestring = dtimestring+tzstr
    endif

ErrorExit:
return, dtimestring

end
