;+
; Copyright © 2008, United States Government as represented by the
; Administrator for The National Aeronautics and Space Administration.
; All Rights Reserved.
;
; AUTHOR:
;       Liang Liao
;       Caelum Research Corp.
;       Code 975, NASA/GSFC
;       Greenbelt, MD 20771
;       Phone: 301-614-5718
;       E-mail: lliao@priam.gsfc.nasa.gov
;
; DESCRIPTION:
; Given matching data arrays of PR and GV radar rain rate, rain type, and
; underlying surface type, produces multiple stratifications of the rain rate
; data based broken out by rain type and surface type in new data arrays.
;  - The size of the input arrays is assumed to be fixed at 75x75 gridpoints (a
;    300x300 km grid of 4 km resolution).
;
; EMAIL QUESTIONS OR COMMENTS TO:
;       <Bob Morris> kenneth.r.morris@nasa.gov
;       <Matt Schwaller> mathew.r.schwaller@nasa.gov
;-

pro commonArea_rain, S_2a25=s_pr, S_2a55=s_gv, $
                     C_2a25=c_pr, C_2a55=c_gv, $
                     T_2a25=t_pr, T_2a55=t_gv, $
                     O_2a25=o_pr, O_2a55=o_gv, $ 
                     L_2a25=l_pr, L_2a55=l_gv, $
                     M_2a25=m_pr, M_2a55=m_gv
                     
common PR_GV, rainType_2a23, rainType_2a54, dbz_1c21, dbz_2a25, dbz_2a55, oceanFlag
common PR_GV_RAIN, surfRain_pr, surfRain_gv
common sample, start_sample,sample_range,num_range,dbz_min,dbz_max 
common sample_rain, RAIN_MIN, RAIN_MAX
common cumulation, s_npt, c_npt, t_npt, o_npt, l_npt, m_npt, $
                   s_pr_tot, s_gv_tot, c_pr_tot, c_gv_tot, $
                   t_pr_tot, t_gv_tot, o_pr_tot, o_gv_tot, $
                   l_pr_tot, l_gv_tot, m_pr_tot, m_gv_tot
 
s_count = 0L
c_count = 0L
t_count = 0L
o_count = 0L
l_count = 0L
m_count = 0L

s_space = fltarr(5625)
c_space = fltarr(5625)
t_space = fltarr(5625)
o_space = fltarr(5625)
l_space = fltarr(5625)
m_space = fltarr(5625)

s_ground = fltarr(5625)
c_ground = fltarr(5625)
t_ground = fltarr(5625)
o_ground = fltarr(5625)
l_ground = fltarr(5625)
m_ground = fltarr(5625)

for i=0, 74 do begin
for j=0, 74 do begin
  
  if (surfRain_pr[i,j] gt (rain_min)) and (surfRain_gv[i,j] gt (rain_min)) and $
     (surfRain_pr[i,j] le (rain_max)) and (surfRain_gv[i,j] le (rain_max)) then begin
     
    if (rainType_2a23[i,j] eq 1) and (rainType_2a54[i,j] eq 1) then begin
      
      s_space[s_count] = surfRain_pr[i,j]
      s_ground[s_count] = surfRain_gv[i,j]
      s_count = s_count + 1
      
      s_pr_tot[s_npt] = surfRain_pr[i,j]
      s_gv_tot[s_npt] = surfRain_gv[i,j]
      s_npt = s_npt + 1
      
    endif
    
    if (rainType_2a23[i,j] eq 2) and (rainType_2a54[i,j] eq 2) then begin
      
      c_space[c_count] = surfRain_pr[i,j]
      c_ground[c_count] = surfRain_gv[i,j]
      c_count = c_count + 1
      
      c_pr_tot[c_npt] = surfRain_pr[i,j]
      c_gv_tot[c_npt] = surfRain_gv[i,j]
      c_npt = c_npt + 1

    endif
    
    t_space[t_count] = surfRain_pr[i,j]
    t_ground[t_count] = surfRain_gv[i,j]
    t_count = t_count + 1
    
    t_pr_tot[t_npt] = surfRain_pr[i,j]
    t_gv_tot[t_npt] = surfRain_gv[i,j]
    t_npt = t_npt + 1
    
; ..............................................

    if (oceanFlag[i,j] eq 0) then begin
      
      o_space[o_count] = surfRain_pr[i,j]
      o_ground[o_count] = surfRain_gv[i,j]
      o_count = o_count + 1
      
      o_pr_tot[o_npt] = surfRain_pr[i,j]
      o_gv_tot[o_npt] = surfRain_gv[i,j]
      o_npt = o_npt + 1
      
    endif
    
    if (oceanFlag[i,j] eq 10) then begin
      
      l_space[l_count] = surfRain_pr[i,j]
      l_ground[l_count] = surfRain_gv[i,j]
      l_count = l_count + 1
      
      l_pr_tot[l_npt] = surfRain_pr[i,j]
      l_gv_tot[l_npt] = surfRain_gv[i,j]
      l_npt = l_npt + 1
      
    endif
    
    if (oceanFlag[i,j] ne 0) and (oceanFlag[i,j] ne 10) then begin
      
      m_space[m_count] = surfRain_pr[i,j]
      m_ground[m_count] = surfRain_gv[i,j]
      m_count = m_count + 1
      
      m_pr_tot[m_npt] = surfRain_pr[i,j]
      m_gv_tot[m_npt] = surfRain_gv[i,j]
      m_npt = m_npt + 1
      
    endif

    
  endif
  
endfor
endfor

if s_count gt 0 then begin
   s_pr = s_space[0:(s_count-1)]
   s_gv = s_ground[0:(s_count-1)]
endif else begin
   s_pr = -99.
   s_gv = -99.
endelse 
 
if c_count gt 0 then begin
   c_pr = c_space[0:(c_count-1)]
   c_gv = c_ground[0:(c_count-1)]
endif else begin
   c_pr = -99.
   c_gv = -99.
endelse

if t_count gt 0 then begin
   t_pr = t_space[0:(t_count-1)]
   t_gv = t_ground[0:(t_count-1)]
endif else begin
   t_pr = -99.
   t_gv = -99.
endelse   

if o_count gt 0 then begin
   o_pr = o_space[0:(o_count-1)]
   o_gv = o_ground[0:(o_count-1)]
endif else begin
   o_pr = -99.
   o_gv = -99.
endelse

if l_count gt 0 then begin
   l_pr = l_space[0:(l_count-1)]
   l_gv = l_ground[0:(l_count-1)]
endif else begin
   l_pr = -99.
   l_gv = -99.
endelse   

if m_count gt 0 then begin
   m_pr = m_space[0:(m_count-1)]
   m_gv = m_ground[0:(m_count-1)]
endif else begin
   m_pr = -99.
   m_gv = -99.
endelse   
   
end

    
    
         
         
       
                
